//
// C++ Interface: metgraph
//
// Description: Meteo Graphs
//
//
// Author: Thomas Sailer <t.sailer@alumni.ethz.ch>, (C) 2014
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifndef METGRAPH_H
#define METGRAPH_H

#include "sysdeps.h"
#include "geom.h"
#include "dbobj.h"
#include "fplan.h"
#include "grib2.h"

class MeteoGraph {
};

class MeteoProfile : public MeteoGraph {
  public:
	MeteoProfile(const FPlanRoute& route = FPlanRoute(*(FPlan *)0),
		     const TopoDb30::RouteProfile& routeprofile = TopoDb30::RouteProfile(),
		     const GRIB2::WeatherProfile& wxprofile = GRIB2::WeatherProfile());

	const FPlanRoute& get_route(void) const { return m_route; }
	void set_route(const FPlanRoute& r = FPlanRoute(*(FPlan *)0)) { m_route = r; }
	const TopoDb30::RouteProfile& get_routeprofile(void) const { return m_routeprofile; }
	void set_routeprofile(const TopoDb30::RouteProfile& rteprof = TopoDb30::RouteProfile()) { m_routeprofile = rteprof; }
	const GRIB2::WeatherProfile& get_wxprofile(void) const { return m_wxprofile; }
	void set_wxprofile(const GRIB2::WeatherProfile& wxprof = GRIB2::WeatherProfile()) { m_wxprofile = wxprof; }

	double get_scaledist(const Cairo::RefPtr<Cairo::Context>& cr, int width, double dist) const;
	double get_scaleelev(const Cairo::RefPtr<Cairo::Context>& cr, int height, double elev) const;
	void draw(const Cairo::RefPtr<Cairo::Context>& cr, int width, int height,
		  double origindist, double scaledist, double originelev, double scaleelev) const;

  protected:
	FPlanRoute m_route;
	TopoDb30::RouteProfile m_routeprofile;
	GRIB2::WeatherProfile m_wxprofile;

	static inline void set_color_sky(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0x76 / 255.0, 0xa3 / 255.0, 0xee / 255.0);
	}

	static inline void set_color_sky(const Cairo::RefPtr<Cairo::LinearGradient>& g) {
		g->add_color_stop_rgb(0.0, 0x76 / 255.0, 0xa3 / 255.0, 0xee / 255.0);
		g->add_color_stop_rgb(1.0, 0x0a / 255.0, 0x37 / 255.0, 0x82 / 255.0);
	}

	static inline void set_color_sky_night(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0x76 / 255.0 * 0.2, 0xa3 / 255.0 * 0.2, 0xee / 255.0 * 0.2);
	}

	static inline void set_color_sky_duskdawn(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0x76 / 255.0 * 0.6, 0xa3 / 255.0 * 0.6, 0xee / 255.0 * 0.6);
	}

	static inline void set_color_sky_duskdawn(const Cairo::RefPtr<Cairo::LinearGradient>& g) {
		g->add_color_stop_rgba(0.0, 0x76 / 255.0 * 0.2, 0xa3 / 255.0 * 0.2, 0xee / 255.0 * 0.2, 0.0);
		g->add_color_stop_rgba(1.0, 0x76 / 255.0 * 0.2, 0xa3 / 255.0 * 0.2, 0xee / 255.0 * 0.2, 1.0);
	}

	static inline void set_color_terrain(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0.31765, 0.21176, 0.13725);
	}

	static inline void set_color_terrain_fill(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgba(0.31765, 0.21176, 0.13725, 0.333);
	}

	static inline void set_color_fplan(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(1., 0., 1.);
	}

	static inline void set_color_marking(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0., 0., 0.);
	}

	static inline void set_color_grid(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0x82 / 255., 0x82 / 255., 0x82 / 255.);
	}

	static inline void set_color_0degisotherm(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0x82 / 255., 0., 0.);
	}

	static inline void set_color_otherisotherm(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0.0, 0x82 / 255., 0.0);
	}

	static inline void set_color_cloud(const Cairo::RefPtr<Cairo::Context>& cr, double cover) {
		cover = std::max(0., std::min(1., cover));
		double gray(1.0 - 0.25 * cover);
		cr->set_source_rgba(gray, gray, gray, 0.5 + 2.0 * cover);
	}

	static inline void set_color_cloud(const Cairo::RefPtr<Cairo::LinearGradient>& g, double pos, double cover) {
		cover = std::max(0., std::min(1., cover));
		double gray(1.0 - 0.25 * cover);
		g->add_color_stop_rgba(pos, gray, gray, gray, 0.5 + 2.0 * cover);
	}

	static inline void set_color_isotach(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(156 / 256.0, 85 / 256.0, 27 / 256.0);
	}

	class LinePoint {
	public:
		LinePoint(uint32_t x = 0, int32_t y = 0, bool dir = false) : m_x(x), m_y(y), m_dir(dir) {}
		uint32_t get_x(void) const { return m_x; }
		int32_t get_y(void) const { return m_y; }
		bool is_dir(void) const { return m_dir; }
		int compare(const LinePoint& x) const;
		bool operator==(const LinePoint& x) const { return !compare(x); }
		bool operator!=(const LinePoint& x) const { return !operator==(x); }
		bool operator<(const LinePoint& x) const { return compare(x) < 0; }
		bool operator<=(const LinePoint& x) const { return compare(x) <= 0; }
		bool operator>(const LinePoint& x) const { return compare(x) > 0; }
		bool operator>=(const LinePoint& x) const { return compare(x) >= 0; }

	protected:
		uint32_t m_x;
		int32_t m_y;
		bool m_dir;
	};

	class CloudPoint {
	public:
		CloudPoint(double dist = 0, int32_t base = 0, int32_t top = 0, double cover = 0)
			: m_dist(dist), m_cover(cover), m_base(base), m_top(top) {}

		double get_dist(void) const { return m_dist; }
		double get_cover(void) const { return m_cover; }
		int32_t get_base(void) const { return m_base; }
		int32_t get_top(void) const { return m_top; }

	protected:
		double m_dist;
		double m_cover;
		int32_t m_base;
		int32_t m_top;
	};

	static void extract_remainder(std::vector<LinePoint>& ln, std::multiset<LinePoint>& lp, uint32_t maxydiff = 1000);
	static std::vector<LinePoint> extract_line(std::multiset<LinePoint>& lp, uint32_t maxydiff = 1000);
	static std::vector<LinePoint> extract_area(std::multiset<LinePoint>& lp, uint32_t maxydiff = 1000);
	static void draw_clouds_old(const Cairo::RefPtr<Cairo::Context>& cr, const std::vector<CloudPoint>& cldpt);
	static void draw_clouds(const Cairo::RefPtr<Cairo::Context>& cr, const std::vector<CloudPoint>& cldpt);
};

class MeteoChart : public MeteoGraph {
  public:
	MeteoChart(const FPlanRoute& route = FPlanRoute(*(FPlan *)0),
		   const GRIB2& wxdb = *(GRIB2 *)0);
	const FPlanRoute& get_route(void) const { return m_route; }
	void set_route(const FPlanRoute& r = FPlanRoute(*(FPlan *)0)) { m_route = r; }
	const GRIB2& get_wxdb(void) const { return *m_wxdb; }
	void set_wxdb(const GRIB2& wxdb = *(GRIB2 *)0) { m_wxdb = &wxdb; clear(); }

	void get_fullscale(int width, int height, Point& center, double& scalelon, double& scalelat) const;
	gint64 get_midefftime(void) const;
	gint64 get_minefftime(void) const { return m_minefftime; }
	gint64 get_maxefftime(void) const { return m_maxefftime; }
	gint64 get_minreftime(void) const { return m_minreftime; }
	gint64 get_maxreftime(void) const { return m_maxreftime; }

	void clear(void);
	void draw(const Cairo::RefPtr<Cairo::Context>& cr, int width, int height,
		  const Point& center, double scalelon, double scalelat, double pressure = 0);

  protected:
	FPlanRoute m_route;
	const GRIB2 *m_wxdb;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxwindu;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxwindv;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxprmsl;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxtemperature;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxtropopause;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldbdrycover;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldlowcover;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldlowbase;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldlowtop;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldmidcover;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldmidbase;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldmidtop;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldhighcover;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldhighbase;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldhightop;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldconvcover;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldconvbase;
	Glib::RefPtr<GRIB2::LayerInterpolateResult> m_wxcldconvtop;
	gint64 m_minefftime;
	gint64 m_maxefftime;
	gint64 m_minreftime;
	gint64 m_maxreftime;
	bool m_gndchart;

	void loaddb(const Rect& bbox, gint64 efftime, double pressure);

	static inline void set_color_fplan(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(1., 0., 1.);
	}

	static inline void set_color_marking(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0., 0., 0.);
	}

	static inline void set_color_temp(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(1., 0., 0.);
	}

	static inline void set_color_qnh(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0., 0., 1.);
	}

	static inline void set_color_wxnote(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(0., 0., 0.);
	}

	static inline void set_color_isotach(const Cairo::RefPtr<Cairo::Context>& cr) {
		cr->set_source_rgb(156 / 256.0, 85 / 256.0, 27 / 256.0);
	}

	class AreaExtract {
	  public:
		AreaExtract(const Glib::RefPtr<GRIB2::LayerInterpolateResult>& lay, float lim, gint64 efftime, double sfc1value);
		AreaExtract(const Glib::RefPtr<GRIB2::LayerInterpolateResult>& layu,
			    const Glib::RefPtr<GRIB2::LayerInterpolateResult>& layv, float lim, gint64 efftime, double sfc1value);
		AreaExtract(const Glib::RefPtr<GRIB2::LayerInterpolateResult>& laycover,
			    const Glib::RefPtr<GRIB2::LayerInterpolateResult>& laybase,
			    const Glib::RefPtr<GRIB2::LayerInterpolateResult>& laytop, gint64 efftime, double pressure);
		static std::pair<float,float> minmax(const Glib::RefPtr<GRIB2::LayerInterpolateResult>& lay, gint64 efftime, double sfc1value);
		static std::pair<float,float> minmax(const Glib::RefPtr<GRIB2::LayerInterpolateResult>& layu,
						     const Glib::RefPtr<GRIB2::LayerInterpolateResult>& layv, gint64 efftime, double sfc1value);
		std::vector<Point> extract_contour(void);
		void extract_contours(const Cairo::RefPtr<Cairo::Context>& cr, const Point& center, double scalelon, double scalelat);

	  protected:
		class ScalarGridPt {
		  public:
			ScalarGridPt();
			ScalarGridPt(const Glib::RefPtr<GRIB2::LayerInterpolateResult>& lay, unsigned int x, unsigned int y,
				     float lim, gint64 efftime, double sfc1value);
			ScalarGridPt(const ScalarGridPt& x, const ScalarGridPt& y, float lim);
			const Point& get_coord(void) const { return m_coord; }
			void set_coord(const Point& c) { m_coord = c; }
			float get_val(void) const { return m_val; }
			bool is_inside(void) const { return m_inside; }
			Point contour_point(const ScalarGridPt& x, float lim) const;

		  protected:
			Point m_coord;
			float m_val;
			bool m_inside;
		};

		class WindGridPt : public ScalarGridPt {
		  public:
			WindGridPt();
			WindGridPt(const Glib::RefPtr<GRIB2::LayerInterpolateResult>& layu,
				   const Glib::RefPtr<GRIB2::LayerInterpolateResult>& layv,
				   unsigned int x, unsigned int y, float lim, gint64 efftime, double sfc1value);
			WindGridPt(const WindGridPt& x, const WindGridPt& y, float lim);
		};

		class CloudGridPt {
		  public:
			static const float mincover = 0.4;

			CloudGridPt();
			CloudGridPt(const Glib::RefPtr<GRIB2::LayerInterpolateResult>& laycover,
				    const Glib::RefPtr<GRIB2::LayerInterpolateResult>& laybase,
				    const Glib::RefPtr<GRIB2::LayerInterpolateResult>& laytop,
				    unsigned int x, unsigned int y, gint64 efftime, float pressure);
			CloudGridPt(const CloudGridPt& x, const CloudGridPt& y, float pressure);
			const Point& get_coord(void) const { return m_coord; }
			void set_coord(const Point& c) { m_coord = c; }
			float get_cover(void) const { return m_cover; }
			float get_base(void) const { return m_base; }
			float get_top(void) const { return m_top; }
			bool is_inside(void) const { return m_inside; }
			Point contour_point(const CloudGridPt& x, float lim) const;

		  protected:
			Point m_coord;
			float m_cover;
			float m_base;
			float m_top;
			bool m_inside;
		};

		template <typename PT> class Grid {
		  public:
			Grid(unsigned int w, unsigned int h);
			PT& operator()(unsigned int x, unsigned int y);
			const PT& operator()(unsigned int x, unsigned int y) const;
			unsigned int get_width(void) const { return m_width; }
			unsigned int get_height(void) const { return m_height; }
			void set_border(void);
			unsigned int get_index(unsigned int x, unsigned int y) const;
			unsigned int get_edgenr(unsigned int x0, unsigned int y0, unsigned int x1, unsigned int y1) const;

		  protected:
			std::vector<PT> m_v;
			unsigned int m_width;
			unsigned int m_height;
		};

		class Edge {
		  public:
			Edge(const Point& st = Point::invalid, unsigned int ste = 0, const Point& en = Point::invalid, unsigned int ene = 0,
			     const Point& mid = Point::invalid) : m_start(st), m_end(en), m_intermediate(mid), m_startedge(ste), m_endedge(ene) {}
			const Point& get_start(void) const { return m_start; }
			const Point& get_end(void) const { return m_end; }
			const Point& get_intermediate(void) const { return m_intermediate; }
			unsigned int get_startedge(void) const { return m_startedge; }
			unsigned int get_endedge(void) const { return m_endedge; }
			int compare(const Edge& x) const;
			bool operator==(const Edge& x) const { return compare(x) == 0; }
			bool operator!=(const Edge& x) const { return compare(x) != 0; }
			bool operator<(const Edge& x) const { return compare(x) < 0; }
			bool operator<=(const Edge& x) const { return compare(x) <= 0; }
			bool operator>(const Edge& x) const { return compare(x) > 0; }
			bool operator>=(const Edge& x) const { return compare(x) >= 0; }

		  protected:
			Point m_start;
			Point m_end;
			Point m_intermediate;
			unsigned int m_startedge;
			unsigned int m_endedge;
		};

		typedef std::set<Edge> edges_t;
		edges_t m_edges;

		template <typename PT> void compute(Grid<PT>& grid, float lim);
	};
};

#endif /* METGRAPH_H */
